
/*
 * $Id: ErrorMgr.cc,v 1.3 1999/04/23 17:26:54 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <errno.h>
#include <string.h>

#include "String.h"
#include "LogComment.h"
#include "StatPing.h"
#include "StatPhaseMgr.h"
#include "ErrorMgr.h"

#include "globals.h"

static ErrorMgr *TheErrMgr = 0;

/* SrcLoc */

SrcLoc::SrcLoc(const char *aFName, int aLineNo):
	theFName(aFName), theLineNo(aLineNo) {
}

ostream &SrcLoc::print(ostream &os) const {
	if (theFName)
		os << theFName << ':' << theLineNo << ": ";
	return os;
}


/* ErrorMgr */

static
void CanonError(int &e) {
	if (e < 0 || e > TheErrMgr->maxErrNo())
		e = TheErrMgr->maxErrNo();
}

void ErrorMgr::Configure(ErrorMgr *mgr) {
	Assert(mgr);
	TheErrMgr = mgr;
}

ErrorMgr::ErrorMgr(int aMaxErrNo): theVerbLvl(1), theMaxErrNo(aMaxErrNo+1),
	theTotCnt(0) {

	theErrCnts.stretch(theMaxErrNo+1);
	theExplDlines.stretch(theMaxErrNo+1);
}

bool ErrorMgr::needExpl(int e) {
	Assert(e >= 0 && e < theErrCnts.capacity());
	if (theVerbLvl > 0 && theErrCnts[e] >= theExplDlines[e]) {
		theExplDlines[e] += 1 + theExplDlines[e]/theVerbLvl;
		return true;
	}
	return false;
}

void ErrorMgr::noteError(int e) {
	Assert(e >= 0 && e <= theMaxErrNo);
	theErrCnts[e]++;
	theTotCnt++;
	CallStat(ThePinger, err(e));
}

void ErrorMgr::reportError(const SrcLoc &loc, int e) {
	Assert(e >= 0 && e <= theMaxErrNo);
	Comment(0) 
		<< loc << "error: "
		<< theErrCnts[e] << '/' << theTotCnt
		<< " (" << e << ") "
		<< explStr(e)
		<< endc;
}

const char *ErrorMgr::explStr(int e) const {
	if (e >= 0 && e <= 255)
		return strerror(e);
	if (e >= theMaxErrNo)
		return "error code too high to make sense";
	return "unknown error code";
}

/* global routines */

void NoteError(int e) {
	Assert(TheErrMgr);
	CanonError(e);
	TheErrMgr->noteError(e);
}

bool ExplainError(int e) {
	Assert(TheErrMgr);
	CanonError(e);
	TheErrMgr->noteError(e);
	return TheErrMgr->needExpl(e);
}

void ReportErrorLoc(const SrcLoc &loc, int e) {
	Assert(TheErrMgr);
	CanonError(e);
	TheErrMgr->noteError(e);
	if (TheErrMgr->needExpl(e))
		TheErrMgr->reportError(loc, e);
}

void FatalErrorLoc(const SrcLoc &loc, int e) {
	Assert(TheErrMgr);
	CanonError(e);
	TheErrMgr->noteError(e);
	TheErrMgr->reportError(loc, e);
	Comment(0) << "last error (" << e << ") was fatal; terminating..." << endc;
	exit(-e);
}
