
/*
 * $Id: TmlModel.cc,v 1.2 1999/04/15 20:35:54 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "Rnd.h"
#include "TmlModel.h"
#include "Oid.h"
#include "gadgets.h"

TmlModel::TmlModel(): theDistDistr(0), theSearchDelta(0) {
}

int TmlModel::pickOid(bool cachable) {
	const int depth = Min(theFuture.capacity(), 1+2*theSearchDelta);
	for (int i = 0; i < depth; ++i) {
		const int oid = theFuture[i];
		if (oid > 0 && CachableOid(oid) == cachable) {
			if (i) // give [0] a chance to survive the shift
				theFuture[i] = theFuture[0];
			return oid;
		}
	}

	return 0; // failed to find an appropriate oid
}

bool TmlModel::addOid(int oid) {
	const int last = theFuture.capacity() - 1;
	const int best = Min((int)theDistDistr->trial(), last);
	const int end = Min(best + theSearchDelta, last);
	for (int i = Max(0, best - theSearchDelta); i <= end; ++i) {
		// XXX: do we need a better conflict resolution?
		if (theFuture[i] <= 0) {
			theFuture[i] = oid;
			return true;
		}
	}
	return false; // failed to find an empty slot near by
}

int TmlModel::curPlanDepth() const {
	int last_busy = 0;
	for (int i = 0; i < theFuture.capacity(); ++i) {
		if (theFuture[i] > 0)
			last_busy = i;
	}
	return last_busy;
}
