
/*
 * $Id: WorldOpts.cc,v 1.3 1999/04/20 23:10:09 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "UniqId.h"
#include "Rnd.h"
#include "ZipfPopMdl.h"
#include "UnifPopMdl.h"
#include "LogComment.h"
#include "WorldOpts.h"

#include "globals.h"


WorldOpts::WorldOpts():
	theType(this,      "world_type <str>",   "part of a URL", "Polygraph"),
	theId(this,        "world_id <str>",     "part of a URL", Id2Str(UniqId::Create())),
	theUrlsFileName(this,"world_urls <file>","file with URLs"),

	theUniqStart(this, "unique_urls <int>",  "append unique suffix (counter) to URLs"),
	theReqOrder(0,     "order <int>",        "request order"),

	theChbRatio(this,  "rep_cachable <ratio>","portion of cachable replies"),
	theHitRatio(this,  "dhr <ratio>",        "document hit ratio"),
	thePopMdl(this,    "pop_model <model>",  "popularity model", new ZipfPopMdl()),
	theTmlDistr(this,  "tmp_loc <distr>",    "distances distr for temporal locality"),
	theTmlDelta(this,  "tmp_loc_delta <int>","fuzziness of temporal locality choices", 1),
	theTmlDepth(this,  "tmp_loc_depth <int>","max depth of tmp locality plan", 500000),

	theExpOidCnt(0,    "exp_oids <int>",     "how many unique oids to expect"),
	thePrefilledCnt(this,"prefilled_cnt <int>","number of already cached oids")

{
	theTmlDistr.argType("num");
}

bool WorldOpts::validate() const {
	if (!thePopMdl)
		cerr << ThePrgName << ": popularity model must be set" << endl;
	else
	if (theChbRatio > 1)
		cerr << ThePrgName << ": rep cachability ratio cannot be higher than 1, got: " << theChbRatio << endl;
	else
	if (theHitRatio > 1)
		cerr << ThePrgName << ": document hit ratio cannot be higher than 1, got: " << theHitRatio << endl;
	else
	if (theChbRatio < theHitRatio)
		cerr << ThePrgName << ": cachability ratio cannot be less than"
			<< " document hit ratio, got: " << theChbRatio << " < " << theHitRatio << endl;
	else
		return OptGrp::validate();
	return false;
}


/* PopMdlOpt */

static const char *valStr(const char *val) { return val ? val : "<none>"; }

PopMdlOpt::PopMdlOpt(OptGrp *aGrp, const char *aName, const char *aDescr, PopModel *def):
	Opt(aGrp, aName, aDescr), theVal(def) {
}

bool PopMdlOpt::parse(const char *name, const char *val) {
	if (val) {
		delete theVal;
		theVal = 0;

		if (!strcmp(val, "zipf")) {
			theVal = new ZipfPopMdl();
		} else
		if (!strcmp(val, "unif")) {
			theVal = new UnifPopMdl();
		}

		if (theVal)
			return true;
	}
	cerr << ThePrgName << ": popularity model (e.g., `zipf') expected for the `" << name << "` option; got `" << valStr(val) << "'" << endl;
	return false;
}

void PopMdlOpt::report() const {
	if (theVal)
		Comment << *theVal;
	else
		Comment << "<none>";
}
