
/*
 * $Id: StatPhase.h,v 1.1.1.5 1999/04/05 19:09:10 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef STAT_PHASE_H
#define STAT_PHASE_H

#include "Assert.h"
#include "StatPing.h"
#include "StatSoft.h"
#include "Progress.h"
#include "histograms.h"

class StatPhaseMgr;

// Accumulates all statistics for a phase in StatPhaseRec and dumps it when needed
// StatPhase terminates itself notifying the manager

class StatPhaseRec: public StatPingRec, public StatSoftRec {
	public:
		StatPhaseRec();

		void store(OLog &log) const;
		void load(ILog &);

		void xactRdSz(Size sz, Time ttl) { theXactRdSz.record(sz); theXactTtlVsSz.record(sz, ttl); }

		StatPhaseRec &operator +=(const StatPhaseRec &r);

		Time duration() const { return StatPingRec::theDuration; }

	protected:
		void duration(Time d) { StatPingRec::theDuration = StatSoftRec::theDuration = d; }

	public: // read-only
		SizeHist theXactRdSz;
		TimeVsSizeHist theXactTtlVsSz;
		Goal theGoal;      // absolute number of required completions
};

class StatPhase: public StatInt, public StatPhaseRec {
	public:
		StatPhase(const char *aName, int aLogTag);

		void goal(const Goal &aGoal) { theGoal = aGoal; }
		const char *name() const { return theName; }

		void start(StatPhaseMgr *aMgr);
		void lock() { theLock++; }
		void unlock() { Assert(--theLock >= 0); }

		void xactEnd() { StatPhaseRec::xactEnd(); checkpoint(); }
		void xactErr() { StatPhaseRec::xactErr(); checkpoint(); }

		void flush();

	protected:
		void checkpoint();
		void report() const;

	protected:
		const char *theName;
		int theLogTag;// log tag
		int theLock;  // will not quit if locked

		StatPhaseMgr *theMgr;
};

#endif
