
/*
 * $Id: shutdown.cc,v 1.2 1999/04/15 20:36:01 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "Socket.h"
#include "Select.h"
#include "String.h"
#include "AlarmClock.h"
#include "LogComment.h"
#include "Progress.h"
#include "SharedOpts.h"
#include "StatPhaseMgr.h"
#include "World.h"
#include "shutdown.h"
//#include "gadgets.h"
#include "globals.h"

class CoolOffTimer: public AlarmUser {
	public:
		CoolOffTimer(Time tout);
		virtual void wakeUp(const Alarm &a);
};

static String TheReason = 0;
static int TheSignal = -1;
static Time TheCoolOffTime;

static CoolOffTimer *CoolingOff = 0; // Cool-off phase has started
static bool CooledOff  = false; // Cool-off phase has completed
static bool DoShutdown = false; // ShutdownReason() has been called
static bool InShutdown = false; // Shutdown() has started, has not finished
static bool IsClean = false;    // Shutdown() has finished

// check various shutdown conditions
bool ShutdownNow() {
	if (DoShutdown)
		return true; // already decided
	else
	if (Socket::Level() >= Socket::TheMaxLevel) {
		Comment << "active file descriptors: " << Socket::Level() << "; limit: " << Socket::TheMaxLevel << endc;
		ShutdownReason("dangerously low on file descriptors");
	} else
	if (TheSignal >= 0) {
		ShutdownReason("got shutdown signal");
	} else
	if (CoolingOff) {
		return false; // do not check goal during cool-off phase
	} else
	if (TheOpts.theGoal && TheProgress >= TheOpts.theGoal) {
		if (TheCoolOffTime > 0 && !CooledOff) {
			// start cooling-off
			CoolingOff = new CoolOffTimer(TheCoolOffTime);
			return false;
		} else {
			Comment << "progress: " << TheProgress << "; goal: " << TheOpts.theGoal << endc;
			ShutdownReason("goal is satisfied");
		}
	} else
		return false;

	return DoShutdown = true;
}

bool ShutdownRequested() {
	return DoShutdown;
}

void ShutdownSignal(int s) {
	TheSignal = s;
	Comment << "got shutdown signal (" << TheSignal << ")" << endc;
}

void CoolOffTime(Time t) {
	TheCoolOffTime = t;
}

void ShutdownReason(const String &reason) {
	if (reason)
		TheReason = reason;
	DoShutdown = true;
}

void Shutdown() {
	if (IsClean || InShutdown)
		return; // nothing to do; was here already?!

	DoShutdown = InShutdown = true;

	Clock::Update(false);

	Comment(2) << "got " << TheProgress.theXactComplCnt
		<< " xactions and " << TheProgress.theErrs.theCnt << " errors" << endc;
	if (TheReason)
		Comment << "shutdown reason: " << TheReason << endc;

	// cleanup
	TheStatPhaseMgr.forceFlush();
	TheLog().close();

	IsClean = true;
	InShutdown = false;
}


/* CoolOffTimer */

CoolOffTimer::CoolOffTimer(Time tout) {
	Comment(1) << "cooling-off for " << tout << endc;
	sleepFor(tout);
}

void CoolOffTimer::wakeUp(const Alarm &a) {
	AlarmUser::wakeUp(a);
	Assert(CoolingOff == this);
	CooledOff = true;
	CoolingOff = 0;
	Comment(1) << "cooled-off" << endc;
	ShutdownNow();
}
