#ifndef X_RND_H
#define X_RND_H

/*
 * $Id: Rnd.h,v 1.2 1999/04/20 23:10:22 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include <math.h>

class ostream;

// simple random generator
// every generator gets its own seed and state
// no virtual stuff as we do not need more than one type of generators
class RndGen {
	public:
		typedef long State;
		static State TheDefSeed;   // default seed

	public:
		RndGen(State aSeed = -1);

		// uniform[0,1)
		double trial() { return ltrial()/2147483648.0; }
		double operator ()() { return trial(); }

		// uniform[0,2147483648)
		long ltrial();

		// equiprob[lo,hi); note that "hi" is excluded
		double operator ()(double lo, double hi) { return lo + (hi-lo)*trial(); }
		int operator ()(int lo, int hi) { return (int)(lo + (hi-lo)*trial()); }

		// change seed
		void seed(State seed);
		void reset() { seed(theSeed); }

	protected:
		State theSeed;  // initial seed
		State theState; // current state
};

// abstract distribution
// allows for sharing of one random generator among several distributions
class RndDistr
{
	public:
		RndDistr(RndGen *aGen): theGen(aGen) {}
		virtual ~RndDistr() {} // may be leaking; we do not delete theGen

		virtual const char *pdfName() const = 0; // type of pdf (e.g., exp)
		RndGen *rndGen() { return theGen; }
		void rndGen(RndGen *aGen) { theGen = aGen; }

		// returns next pseudo random number
		double operator ()() { return trial(); }
		virtual double trial() = 0;

		virtual double mean() const = 0; // mean
		virtual double sdev() const = 0;  // standard deviation
		double var() const { return sdev()*sdev(); }  // variance

		virtual ostream &print(ostream &os) const;

	protected:
		RndGen *theGen;
};

inline
ostream &operator <<(ostream &os, const RndDistr &d) { return d.print(os); }

#endif
